﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Reflection;
using System.Text;
using System.Xml;
using log4net.Repository.Hierarchy;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Webhook_Catcher.Hub;
using Webhook_Catcher.Models.Home;
using Webhook_Catcher.Models.Webhook;

namespace Webhook_Catcher.Controllers
{
    public class HomeController : Controller
    {
        private readonly IConfiguration _configuration;
        private readonly IHubContext<WebhookNotificationHub> _hubContext;
        private readonly IUserManager _userManager;
        private readonly ILogger<HomeController> _logger;
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        public HomeController(IConfiguration configuration, IHubContext<WebhookNotificationHub> hubcontext, IUserManager userManager, ILogger<HomeController> logger)
        {
            _configuration = configuration;
            _hubContext = hubcontext;
            _userManager = userManager;
            _logger = logger;
        }

        public IActionResult Index()
        {
            XmlDocument log4NetConfig = new XmlDocument();

            // Loading the configuration
            log4NetConfig.Load(System.IO.File.OpenRead("log4net.config"));
            var repo = log4net.LogManager.CreateRepository(Assembly.GetEntryAssembly(),
                typeof(log4net.Repository.Hierarchy.Hierarchy));

            // Creating the file configuration
            log4net.Config.XmlConfigurator.Configure(repo, log4NetConfig["log4net"]);
            log.Error("entered Index");

            ViewData["NotLoggedIn"] = true;
            
            return View("Index");
        }
        
        [HttpPost]
        public IActionResult LogIn(string username, string password)
        {
            log.Error("entered Logging");
            
            //creating HTTP client to send a request for token
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("Accept", "application/json");

            //downloading the path to OCR API
            string ocrApi = _configuration["OCRRequestAPI:Url"];
            var uri = $"{ocrApi}api/auth/token";
            log.Error(uri);

            //sending the request
            var loginModel = new {
                username,
                password
            };
            var jsonInString = JsonConvert.SerializeObject(loginModel);
            var response = client.PostAsync(uri, new StringContent(jsonInString, Encoding.UTF8, "application/json")).Result;
            
            log.Error(response);
            
            //error view if there is one
            if (response.IsSuccessStatusCode == false)
            {
                return View("Error", new ErrorViewModel("Operation has failed - wrong login data."));
            }
            
            //receiving the response and converting it
            var contents = response.Content.ReadAsStringAsync().Result;
            var jsonContent = JsonConvert.DeserializeObject<AccountModel>(contents);
            
            //error view if there is no token
            if (jsonContent?.token == null)
            {
                return View("Error", new ErrorViewModel("Operation has failed - no token."));
            }
            
            HttpContext.Session.SetString("token", jsonContent.token);

            return LoggedIn(username);
        }
        
        public IActionResult LoggedIn(string username)
        {
            ViewData["username"] = username;
            ViewData["domainLink"] = _configuration["WebhookCatcher:Url"];
            
            _userManager.RemoveUserConnection(username);

            var documentDefinitions = GetDocumentDefinitions();
            if (documentDefinitions == null)
            {
                return View("Error", new ErrorViewModel("Operation has failed - no document definitions."));
            }
            log.Error(username);
            
            return View("UserMenu", new DocumentUploadModel(documentDefinitions));
        }

        [HttpGet]
        public List<DocumentDefinitionsModel> GetDocumentDefinitions()
        {
            //creating HTTP client to receive available document definitions
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("Accept", "application/json");

            //downloading the path to OCR API
            string ocrApi = _configuration["OCRRequestAPI:Url"];
            var uri = $"{ocrApi}api/connector/definitions";
            
            //adding bearer token to the request
            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", HttpContext.Session.GetString("token"));

            //sending the request
            var response = client.GetAsync(uri).Result;

            log.Error("definitions:" + response);

            if (response.IsSuccessStatusCode)
            {
                //receiving the response and converting it
                var contents = response.Content.ReadAsStringAsync().Result;
                var jsonContent = JsonConvert.DeserializeObject<List<DocumentDefinitionsModel>>(contents);
                return jsonContent;
            }

            return null;
        }
        
        [HttpGet]
        public IActionResult SeeAllDocuments()
        {
            //creating HTTP client to download a list of documents
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("Accept", "application/json");

            //downloading the path to OCR API
            string ocrApi = _configuration["OCRRequestAPI:Url"];
            var uri = $"{ocrApi}api/connector/documents";
            
            //adding bearer token to the request
            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", HttpContext.Session.GetString("token"));

            //sending the request
            var response = client.GetAsync(uri).Result;

            log.Error("all documents:" + response);
            
            if (!response.IsSuccessStatusCode)
            {
                return View("Error", new ErrorViewModel("Operation has failed."));
            }

            //receiving the response and converting it
            var contents = response.Content.ReadAsStringAsync().Result;
            var jsonContent = JsonConvert.DeserializeObject<List<DocumentModel>>(contents);
            
            //sorting the list from newest to oldest
            List<DocumentModel> SortedList = jsonContent.OrderByDescending(o=>o.CreationDate).ToList();

            return View("UserDocuments", SortedList);
        }
        
        [HttpPost]
        public IActionResult UploadDocument(string documentType, string categoryCode, IFormFile file, bool recognizeTables, string username)
        {
            var documentUploadModel = new DocumentUploadModel();
            
            //assigning form values
            documentUploadModel.CategoryCode = categoryCode;
            documentUploadModel.DocumentType = documentType;
            documentUploadModel.File = file;
            documentUploadModel.RecognizeTables = recognizeTables;
            //downloading the path to webhook controller
            string catcherLink = _configuration["WebhookCatcherForWebhook:Url"];
            documentUploadModel.NotificationURl = $"{catcherLink}Webhook/CatchWebhook/";
            
            //creating HTTP client to send request for uploading document
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("Accept", "application/json");
            
            //downloading the path to OCR API
            string ocrApi = _configuration["OCRRequestAPI:Url"];
            var uri = $"{ocrApi}api/connector/documents";
            
            //adding bearer token to the request
            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", HttpContext.Session.GetString("token"));
            
            //adding data to the request
            var formData = new MultipartFormDataContent();

            foreach (var prop in documentUploadModel.GetType().GetProperties())
            {
                var value = prop.GetValue(documentUploadModel);
                if (value is FormFile)
                {
                    //processing the added file
                    var documentFile = value as FormFile;
                    var sc = new StreamContent(file.OpenReadStream());
                    
                    //adding content-type for correct file displaying
                    sc.Headers.Add("Content-Type", "application/pdf");
                    sc.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data") { Name = prop.Name, FileName = documentFile.FileName };
                    formData.Add(sc, prop.Name, documentFile.FileName);
                }
                else if (value is string)
                {
                    var stringValue = value as string;
                    formData.Add(new StringContent(stringValue), prop.Name);
                }
                else if (value is null)
                {
                    //sending null to the database
                }
                else
                {
                    formData.Add(new StringContent(JsonConvert.SerializeObject(value)), prop.Name);
                }
            }
            log.Error("uploading:" + formData);

            //receiving the response
            var response = client.PostAsync(uri, formData).Result;
            log.Error("uploading:" + response);

            //error view if there is one
            if (response.IsSuccessStatusCode == false)
            {
                return View("Error", new ErrorViewModel("Operation has failed - invalid document data."));
            }
            
            //converting the response
            var contents = response.Content.ReadAsStringAsync().Result;
            var result = JsonConvert.DeserializeObject<DocumentUploadedModel>(contents);

            //adding the token and the id of the uploaded file to the connected user
            var targetClients = _userManager.GetConnectedClients().Where(user => user.Username == username).ToArray();
            if (targetClients != null)
            {
                foreach (var target in targetClients)
                {
                    target.Token = HttpContext.Session.GetString("token");
                    target.UploadedDocument = result.Id;
                }
            }

            return DocumentPending(result);
        }

        public IActionResult DocumentPending(DocumentUploadedModel documentInfo)
        {
            return View("DocumentPending", documentInfo);
        }

    }
}
