﻿using System;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.SignalR;

namespace Webhook_Catcher.Hub
{
    public class WebhookNotificationHub : Microsoft.AspNetCore.SignalR.Hub
    {
        private readonly IUserManager _userManager;

        public WebhookNotificationHub(IUserManager userManager)
        {
            _userManager = userManager;
        }
        
        //function that sends a notification to js hub to show a button for viewing document result
        public async Task ReceiveWebhook(int id)
        {
            await Clients.All.SendAsync("ReceiveWebhook", id);
        }
        
        public override Task OnConnectedAsync()
        {
            //adding data to the current user after each view load
            var httpContext = this.Context.GetHttpContext();
            string username = httpContext.Request.Query["username"];
            var currentUser = new HubClient(Context.ConnectionId, username);

            var targetClients = _userManager.GetConnectedClients().Where(client => client.Username == username && !string.IsNullOrEmpty(client.Token)).ToArray();
            if (targetClients != null)
            {
                foreach (var target in targetClients)
                {
                    currentUser.Token = target.Token;
                    currentUser.UploadedDocument = target.UploadedDocument;
                }
            }
            _userManager.KeepUserConnection(currentUser);
            
            return base.OnConnectedAsync();
        }
        
        public override Task OnDisconnectedAsync(Exception exception)
        {
            return base.OnDisconnectedAsync(exception);
        }
    }
}
